#
# $Source: /home/nlfm/Working/Zircon/Released/lib/RCS/confTab.tcl,v $
# $Date: 1997/09/18 16:26:36 $
# $Revision: 1.18.1.3 $
#
# ----------------------------------------------------------------------
#   AUTHOR:  Lindsay Marshall <lindsay.marshall@newcastle.ac.uk>
# ----------------------------------------------------------------------
# Copyright 1995 The University of Newcastle upon Tyne (see COPYRIGHT)
# ======================================================================
#
#
package provide zircon 1.18
#
proc configWin {net} {
    global lastConf

    set lastConf($net) Nicknames
    set w .@conf$net
    toplevel $w
    wm title $w "Configure Netspace [$net name]"
    frame $w.btn
    foreach x {revert apply save dismiss} {
	button $w.btn.$x -text [trans $x] -command "conf$x $net"
    }
    evenGrid $w.btn column 0 3
    grid $w.btn.revert $w.btn.apply $w.btn.save $w.btn.dismiss -sticky ew
    canvas $w.page -highlightthickness 0 -width 700
    drawTabs $net
    frame $w.f -relief flat -height 400
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1
    grid $w.page -sticky ew
    grid $w.f -sticky nsew
    grid $w.btn -sticky ew
    tabShow $net $lastConf($net)
}
#
proc drawTabs {net} {
    global lastConf
    set w .@conf$net.page
    set normalColor [$w cget -selectbackground]
    set activeColor [$w cget -background]
    set margin 6
    set x 2
    set maxh 0
    foreach name {Nicknames IRCNames Servers Channels Messages Notices Chat DCC \
      Friends Ignores Info} {
	set id [$w create text [expr {$x+$margin+2}] [expr {-0.5*$margin}]\
	  -anchor sw -text [trans $name] -tags $name]
	set bbox [$w bbox $id]
	set wd [expr {[lindex $bbox 2]-[lindex $bbox 0]}]
	set ht [expr {[lindex $bbox 3]-[lindex $bbox 1]}]
	if {$ht > $maxh} { set maxh $ht	}
	set xm [expr {$x + $margin}]
	set hm [expr {-$ht-$margin}]
	$w create polygon 0 0 $x 0 $xm $hm [expr {$xm+$wd}] $hm \
	  [expr {$x+$wd+2*$margin}] 0 2000 0 2000 10 0 10 -outline black \
	  -fill $normalColor -tags [list $name tab tab-$name]
	$w raise $id
	$w bind $name <ButtonPress-1> "tabShow $net $name"
	incr x [expr {$wd+2*$margin}]
    }
    set height [expr {$maxh+2*$margin}]
    $w move all 0 $height
    $w configure -width $x -height [expr {$height+4}]
    $w itemconfigure tab-$lastConf($net) -fill $activeColor
    $w raise $lastConf($net)
}
#
proc tabShow {net pane} {
    global t option pref lastConf
    set w .@conf$net
    $w.page itemconfigure tab -fill [$w.page cget -selectbackground]
    $w.page itemconfigure tab-$pane -fill [$w.page cget -background]
    $w.page raise $pane
    set f $w.f
    catch {destroy $f.chld}
    frame $f.chld
    conf$pane $net $f.chld
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1
    grid $f.chld -sticky nsew
    set lastConf($net) $pane
    switch {} [$net confChange] { confClean $net }
    update idletasks
}
#
proc confrevclean {net which} {
    catch {uplevel #0 unset new$net}
    switch $which {
    Servers {
	global newSv delSv
	foreach v $newSv($net) { $v delete }
	unset newSv($net) delSv($net)
	Server :: cleanup new
    }

    Channels {
	global newChn delChn
	foreach v $newChn($net) {
	    if ![$v sys] { $v delete }
	}
	Channel :: cleanup $net
	unset newChn($net) delChn($net)
    }

    Friends {
	global newFrd delFrd
	foreach v $newFrd($net) { $v delete }
	unset newFrd($net) delFrd($net)
    }
    }
}
#
proc confdismiss {net} {
    catch {uplevel #0 unset new$net}
    switch {} [$net confChange] {} default {
	if ![askUser SAVECONF {Save Changes} "You have unsaved changes. Really dismiss?"] return
    }
    killWindow .@conf$net
    foreach x [$net confChange] { confrevclean $net $x }
}
#
proc confrevert {net} {
    global lastConf
    confrevclean $net $lastConf($net)
    conf$lastConf($net) $net
    confClean $net
}
#
proc confsave {net} {
    global lastConf
    confCopyBack $net $lastConf($net)
    saverc $net
    confInit $net $lastConf($net)
    confClean $net
}
#
proc confapply {net} {
    global lastConf
    confCopyBack $net $lastConf($net)
    confInit $net $lastConf($net)
    confCleaner $net
}
#
proc confDirty {net} {
    global lastConf
    $net configure +confChange lastConf($net)
    set win .@conf$net
    foreach bt {revert apply save} { $win.btn.$bt configure -state normal }
}
#
proc confCleaner {net} {
    global lastConf
    uplevel #0 listkill ${net}(confChange) lastConf($net)
    set win .@conf$net
    foreach bt {apply} { $win.btn.$bt configure -state disabled }
}
#
proc confClean {net} {
    global lastConf
    uplevel #0 listkill ${net}(confChange) lastConf($net)
    set win .@conf$net
    foreach bt {revert apply save} { $win.btn.$bt configure -state disabled }
}
#
proc confDABtns {net args} {
    global cbwstate
    set win .@conf$net
    foreach bt {revert apply save dismiss} {
	set cbwstate($net,$bt) [$win.btn.$bt cget -state]
	$win.btn.$bt configure -state disabled
    }
}
#
proc confRABtns {net args} {
    global cbwstate
    set win .@conf$net
    foreach bt {revert apply save dismiss} {
	switch {} [$net confChange] { set st $cbwstate($net,$bt) } \
	  default { set st normal }
	$win.btn.$bt configure -state $st
	catch {unset cbwstate($net,$bt)}
    }
}
